//*******************************************************************************
//  MSP430 DIr169 ADC12 Demo - Autoscan Temp, Vcc/2, Ax
//
//  Description; ADC12 continuously converts Temp, Vcc/2 and all Ax channels.
//  Timer_B1 triggers a conversion 16 times per second with a sample time of
//  (TBCCR0-TBCCR1)*TBCLK ~= 3msec. VREF used for temperature and Vcc/2 samples,
//  Vcc used for Ax samples. ADC12 interrupts occur on ADC12MEM7 in order to
//  disable the internal 2.5V reference for Ax samples and again on ADC12MEM15
//  to re-enable the internal reference for temperature and Vcc/2 samples.
//  The CPU is normally in LPM3. 
//  MCLK = SMCLK = default DCO ~ 800k, ACLK = 32768Hz
//
//                 MSP430F169
//              -----------------
//          /|\|              XIN|-  
//           | |                 | 32768Hz
//           --|RST          XOUT|-
//             |                 |
//  Scope <--  |TB1         P6.0 |<-- Analog Input
//             |            P6.1 |<-- Logic Lo
//             |            P6.2 |<-- Logic Lo
//             |            P6.3 |<-- Logic Lo
//             |            P6.4 |<-- Logic Hi
//             |            P6.5 |<-- Logic Hi
//             |            P6.6 |<-- Logic Hi
//             |            P6.7 |<-- Logic Hi
//
//  Z. Albus
//  Texas Instruments, Inc
//  October 2003
//  Built with IAR Embedded Workbench Version: 3.0B
//*******************************************************************************
#include  <msp430x16x.h>

void main(void)
{  
  WDTCTL = WDTPW + WDTHOLD;                   // Stop WDT 
  P1DIR = 0xCB;                               // Init P1.x
  P1OUT = 0x08;                               // 
  P2DIR = 0xFF;                               // Init P2.x
  P2OUT = 0x00;                               // 
  P3DIR = 0x75;                               // Init P3.x
  P3OUT = 0x00;                               // 
  P4SEL |= 0x02;                              // TB1 Out: ADC12 Trigger
  P4DIR = 0xFF;                               // Init P4.x
  P4OUT = 0x00;                               // 
  P5DIR = 0xFF;                               // Init P5.x
  P5OUT = 0x00;                               // 
  P6SEL |= 0x01;                              // P6.0 = analog input
  P6DIR = 0xFE;                               // Init P6.x
  P6OUT = 0xF0;                               // Set P6.4-7 to logic Hi

  TBCTL = TBSSEL0 + TBCLR;                    // TBCLK = ACLK, clear TBR
  TBCCTL1 = OUTMOD_3;                         // TRCCR1 Set/Reset
  TBCCR0 = 2048;                              // ADC12 sample rate: TBCLK/TBCCR0
  TBCCR1 = 1948;                              // t_sample = 100 TBCLKs ~= 3ms

  ADC12CTL0 = REF2_5V + MSC + ADC12ON;        // 2.5V Ref, ADC12 on
  ADC12CTL1 = CSTARTADD_8 + SHS_3 + CONSEQ_3; // Start MEM8, TB1, Rpt Seq
  ADC12MCTL0 = INCH_10 + SREF_1;              // Select Temp, VREF
  ADC12MCTL1 = INCH_10 + SREF_1;              // Select Temp, VREF
  ADC12MCTL2 = INCH_10 + SREF_1;              // Select Temp, VREF
  ADC12MCTL3 = INCH_11 + SREF_1;              // Select Vcc/2, VREF
  ADC12MCTL4 = INCH_12 + SREF_1;              // Select Vcc/2, VREF
  ADC12MCTL5 = INCH_13 + SREF_1;              // Select Vcc/2, VREF
  ADC12MCTL6 = INCH_14 + SREF_1;              // Select Vcc/2, VREF
  ADC12MCTL7 = INCH_15 + SREF_1;              // Select Vcc/2, VREF
  ADC12MCTL8 = INCH_0;                        // Select A0, Vcc/Vss
  ADC12MCTL9 = INCH_2;                        // Select A2, Vcc/Vss
  ADC12MCTL10 = INCH_4;                       // Select A4, Vcc/Vss
  ADC12MCTL11 = INCH_6;                       // Select A6, Vcc/Vss
  ADC12MCTL12 = INCH_1;                       // Select A1, Vcc/Vss
  ADC12MCTL13 = INCH_3;                       // Select A3, Vcc/Vss
  ADC12MCTL14 = INCH_5;                       // Select A5, Vcc/Vss
  ADC12MCTL15 = INCH_7;                       // Select A7, Vcc/Vss
  ADC12IE = 0x8080;                           // Enable IFGs for ADC12MEM7/15

  ADC12CTL0 |= ENC;                           // Enable conversions
  TBCTL |= MC0;                               // Start Timer_B in up mode
  
  for (;;)
  {
    _BIS_SR(LPM3_bits + GIE);                 // Enter LPM3 with ints
  }
}

#pragma vector=ADC_VECTOR
__interrupt void ADC12ISR (void)
{ 
  switch( ADC12IV )
  {
    case 0x14: ADC12IFG &= ~0x80;             // Clear IFG in SW
               ADC12CTL0 &= ~ENC;             // Clear ENC
               ADC12CTL0 &= ~REFON;           // Turn off VREF
               ADC12CTL0 |= ENC;              // Re-enable conversions
               break;
    case 0x24: ADC12MEM15;                    // Get data (clears IFG)
               ADC12CTL0 &= ~ENC;             // Clear ENC
               ADC12CTL0 |= REFON + ENC;      // Re-enable VREF & conversions
               break;
  }
}
