//*******************************************************************************
//  MSP-FET430F16x - Test Stack Overflow
//
//  Procedure:
//  * Define a stack area of xxx bytes
//  * Write a dedicated value at the end of the stack area
//  * Use an ISR to increase the SP to the end of the stack
//    (The ISR should be triggered by pressing a button.)
//
//  Comment:
//  Every time the button is pressed the ISR will be executed and
//  the return address and status regesiter is pushed onto the stack.
//  Nested interrupts are enabled by setting EINT immediately after
//  proccesing the interrupt. Wait some time in each interrupt so 
//  the user is able to nest several interrupts.
//  Result - the SP will overwrite the wSetBreakpointOn value.
//  This might be used to trigger on (breakpoint)
//  A LED (flashing) is used to signal main() activities. While
//  the ISR is active, flashing will stop.
//
//  After 50 executions of the main loop a read attempt to
//  boot ROM is performed - indicated by switching on D2.
//  This could be used for range triggers on read from a reserved
//  memory location.
//
//  After 100 executions of the main loop, a write to
//  memory is done - indicated by switching D2 off.
//  This could be used for range triggers on a write.
//
//  The loop counter is reset at 110 and restarts from 0.
//
//  T. Mitnacht / S. Schauer
//  Texas Instruments, Inc
//  September 2003
//  Built with IAR Embedded Workbench Pro v2.21A
//
//  Hardware:
//   - Switches at P1.4 and P1.5 (default high)
//   - LED at P1.0 and P1.1 (output 1 for on)
//
//******************************************************************************

#include  <msp430x16x.h>

#define TRUE  (1==1)
#define FALSE (!TRUE)

// Variable Defintions
__no_init unsigned int wSetBreakpointOn @ 0x09A0; // Locate in RAM
                                                  // SP located @ 0x09FE
__no_init unsigned int wDataInFlash @ 0xF000;     // Locate in Flash

__no_init unsigned int wNoData @ 0x0C00;          // Locate in BOOT ROM

__no_init volatile unsigned int wTemp;            // Assign temp var

volatile unsigned int wLoopCounter = 0;           // Processing loop counter

volatile unsigned int wFlags = 0;                 // Flags for push buttons

// Function Prototypes
void Delay(unsigned int Value);

void main(void)
{
  unsigned int i = 0;

  WDTCTL = WDTPW + WDTHOLD;                       // Stop watchdog timer

  P1DIR = 0xCB;                                   // Init P1.x
  P1OUT = 0x08;                                   // 
  P1IE  |= 0x30;                                  // Enable S1 + S2 ints
  P2DIR = 0xFF;                                   // Init P2.x
  P2OUT = 0x00;                                   // 
  P3DIR = 0x75;                                   // Init P3.x
  P3OUT = 0x00;                                   // 
  P4DIR = 0xFF;                                   // Init P4.x
  P4OUT = 0x00;                                   // 
  P5DIR = 0xFF;                                   // Init P5.x
  P5OUT = 0x00;                                   // 
  P6SEL |= 0x01;                                  // P6.0 = analog input
  P6DIR = 0xFE;                                   // Init P6.x
  P6OUT = 0x00;                                   // 
  
  wSetBreakpointOn = 0xA0A0;                      // Set to dedicated value
  
  _EINT();                                        // Enable interrupts
  
  while(TRUE)
  {
     wLoopCounter++;
     P1OUT ^= 0x01;                               // Toggle P1.0
     Delay(10000);
     i++;
     if ( i == 50)
     {
        P1OUT |= 0x02;                            // Set P1.1
        wTemp = wNoData;                          // Should create a Break
                                                  // (no RD from BootROM allowed)
     }
     if ( i == 100)
     {
        P1OUT &= ~0x02;                           // Clear P1.1
        wDataInFlash++;                           // Should create a Break
                                                  // (no WR in Flash allowed)
     }
     if ( i == 110)
     {
        i = 0;
     }
  }
}

void Delay(unsigned int Value)
{
   volatile unsigned int j;
   
   for(j=Value; j>0; j--);
}

#pragma vector=PORT1_VECTOR
__interrupt void isr_port1_button()
{
   volatile unsigned int k;                       // Delay loop counter, use volatile
                                                  // to aviod optimization 
   // S1 pressed?
   if (P1IFG & 0x10) 
    wFlags |= 0x01;                               // Set flag if button is pressed
   else
    wFlags &= ~0x01;                              // Clear otherwise

   // S2 pressed?
   if (P1IFG & 0x20) 
     wFlags |= 0x02;                              // Set flag if button is pressed
   else 
     wFlags &= ~0x02;                             // Clear otherwise

   P1IFG &= ~0x30;                                // Clear interrupt flags
   _EINT();                                       // Enable ints for nesting

   for(k=0; k<0xFFFF; k++);                       // Allow time for nested ISR entry
}
