//******************************************************************************
//   DIr169 Demo - Multimaster I2C With USART1
//
//   Description: With a partner, you will create a multimaster I2C system
//   Two DIr169 boards will each be connected to a computer through the RS232
//   port. The two DIr169 boards will be connected to each other with the I2C
//   ports. When the system is working each of you will be able to type in a 
//   terminal window on your computer and see the characters you are typing 
//   in the terminal window of the other computer.
//
//   The RS232 settings are 2400 baud, N81. The code is normally in LPM3 with
//   both the UART and the I2C ready to receive a charcter. If A UART character
//   comes in, the UART receive interrupt occurs and the code configures the 
//   I2C for master transmiter and transmits the character on the I2C bus. It 
//   then goes back into receive-ready mode for both the UART and the I2C.  If
//   an I2C character comes the I2C receive ready interrupt occurs and the code
//   transmits the character out the UART.  In case arbitration is lost, the 
//   code echoes back the character it was supposed to send via I2C to the UART
//   so the user can see that arbitration was lost.
//
//   
//                                 /|\  /|\
//      COMPUTER      MSP430F169   10k  10k   MSP430F169      COMPUTER
//      --------      ----------    |    |    ----------      --------
//     |        |    |      P3.1|<--|----+-->|P3.1      |    |        |
//     |      TX|--->|URXD1     |   |        |     URXD1|<---|TX      |
//     |      RX|<---|UTXD1     |   |        |     UTXD1|--->|RX      |
//     |        |    |      P3.3|<--+------->|P3.3      |    |        |
//     |        |    |          |            |          |    |        |
//
//
//  M.Mitchell
//  Texas Instruments, Inc
//  November 2003
//  Built with IAR Embedded Workbench Version: 2.20A
//******************************************************************************

#include  "msp430x16x.h"
void I2C_TX(void);                      // Function prototypes

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
  UCTL1 = CHAR;                         // 8-bit character
  UTCTL1 = SSEL0;                       // UCLK = ACLK
  UBR01 = 0x0D;                         // 32k/2400 - 13.65
  UBR11 = 0x00;                         
  UMCTL1 = 0x6B;                        // Modulation
  ME2 |= UTXE1 + URXE1;                 // Enable USART1 TXD/RXD
  IE2 |= URXIE1;                        // Enable USART1 RX interrupt
  P1OUT = 0x3C;                         // LEDx off, disable audio inp, 
                                        // IrDA transceiver in standby
                                        // IrDA RXD pin high
  P1DIR = 0xff;                         // Unused pins as output  
  P2DIR = 0xff;                         // Unused pins as output  
  P3SEL = 0xCA;                         // USART1 and I2C pin option select
  P3DIR = 0x20;                         // P3.6 = output direction
  P4DIR = 0xff;                         // Unused pins as output  
  P5DIR = 0xff;                         // Unused pins as output
  P6DIR = 0xff;                         // Unused pins as output
 
///// TASK1
  U0CTL |= ?????;                       // Recommended I2C init procedure
  U0CTL &= ?????;                       // Recommended I2C init procedure
///// END TASK1

  I2CTCTL |= I2CSSEL1;                  // SMCLK, for I2C

///// TASK2
  I2COA = ?????;                        // Own address
  I2CSA = ?????;                        // Write to I2C address
///// END TASK2

  I2CIE = RXRDYIE+ALIE+NACKIE+ARDYIE;   // Enable used I2C interrupts
  
  U0CTL |= I2CEN;                       // Enable I2C

  for (;;)                              
  {
  _BIS_SR(LPM3_bits+GIE);               // Enter LPM3 with interrupts
  while (I2CBUSY & I2CDCTL);            // I2C ready?
  I2C_TX();                             // Trasmit received char over I2C
  }
}

void I2C_TX(void)
{
  U0CTL |= MST;                         // Master mode
  I2CNDAT = 0x01;                       // Write one byte

///// TASK3
  I2CTCTL |= ?????;                     // Initiate transfer
///// END TASK3

  I2CDRB = RXBUF1;                      // Copy RXBUF1 to I2CDRB  
}

// Common ISR for I2C Module
#pragma vector=USART0TX_VECTOR
    __interrupt void I2C_ISR(void)
{
  switch(I2CIV)
  {
   case  2:                             // Arbitration lost
     TXBUF1 = RXBUF1;                   // Echo
     I2CTCTL &= ~I2CTRX;                // Clear transmit mode   
     break;
   case  4:                             // No Acknowledge
     U0CTL &= ~MST;                     // Clear Master mode
     I2CTCTL &= ~I2CTRX;                // Clear transmit mode
     break;
   case  6: break;                      // Own Address
   case  8:                             // Register Access Ready
     I2CTCTL &= ~I2CTRX;                // Clear transmit mode   
     break;
   case 10:                             // Receive Ready
     while (!(IFG2 & UTXIFG1));         // USART1 TX buffer ready?
     I2CTCTL &= ~I2CTRX;                // Clear trasmit mode
     TXBUF1 = I2CDRB;                   // Copy I2C RX data to TXBUF1
     break;
   case 12: break;                      // Transmit Ready
   case 14: break;                      // General Call
   case 16: break;                      // Start Condition
  }
}  

// UART1 RX ISR will for exit from LPM3 in Mainloop
#pragma vector=USART1RX_VECTOR
__interrupt void usart1_rx(void)
{
  _BIC_SR_IRQ(LPM3_bits);               // Clear LPM3 bits from 0(SR)
}
