//******************************************************************************
//  MSP430 DIr169 UART Demo - USART1 UART 57600 ULP Echo ISR, 32kHz ACLK & DCO
//
//  Description; Echo a received character, USART1 RX ISR at high-speed used 
//  with ultra-low power techniques. Normal operation in LPM3, Set_DCO 
//  subroutine needs to be called to configure DCO which is used for UART baud 
//  generation. On valid RX character, character echoed  back. Use start-bit 
//  edge detect - URXSE - to automatically (re)enable DCO and trigger ISR. ISR
//  must make sure DCO clock source remains enabled for the UART to receive 
//  full character. Software also needs to make sure a character has been 
//  completely TX'ed before entering LPM3, which disables DCO and ACLK.  
//  ACLK = LFXT1/8 = 32768/8,  MCLK = SMCLK = UCLK0 = DCOCLK  = 1048576
//  Baud rate divider with 1048576hz = 1048576Hz/57600 ~ 18 (0012h)
//  //*An external 32kHz watch crystal on XIN,XOUT is required for ACLK*//	  
//
//              
//               MSP430F169   
//             -----------------
//         /|\|              XIN|-  
//          | |                 | 32768Hz 
//          --|RST          XOUT|-
//            |                 |
//            |             P3.6|------------> 
//            |                 | 57600 - 8N1
//            |             P3.7|<------------
//
//       
//  Z. Albus
//  Texas Instruments, Inc
//  October 2003
//  Built with IAR Embedded Workbench Version: 3.0B
//******************************************************************************

#include <msp430x16x.h>

#define DELTA 256                             // Target DCO = DELTA*(4096) = 
                                              // = 1048576

void Set_DCO (void);

void main(void)
{
  unsigned int i;

  WDTCTL = WDTPW + WDTHOLD;                   // Stop WDT
  BCSCTL1 |= DIVA1 + DIVA0;                   // ACLK = LFXT1CLK/8
  P1DIR = 0xCB;                               // Init P1.x
  P1OUT = 0x08;                               // 
  P2DIR = 0xFF;                               // Init P2.x
  P2OUT = 0x00;                               // 
  P3SEL |= 0xC0;                              // P3.6,7 = USART1 TXD/RXD
  P3DIR = 0x75;                               // Init P3.x
  P3OUT = 0x00;                               // 
  P4DIR = 0xFF;                               // Init P4.x
  P4OUT = 0x00;                               // 
  P5DIR = 0xFF;                               // Init P5.x
  P5OUT = 0x00;                               // 
  P6SEL |= 0x01;                              // P6.0 = analog input
  P6DIR = 0xFE;                               // Init P6.x
  P6OUT = 0x00;                               // 
  
  for (i = 0xFFFF; i > 0; i--);               // Delay for XTAL to settle
  Set_DCO();                                  // Set DCO (ACLK must be active!)

  ME2 |= UTXE1 + URXE1;                       // Enabled USART1 TXD/RXD
  UCTL1 |= CHAR;                              // 8-bit character, SWRST=1
  UTCTL1 |= SSEL1 + URXSE;                    // UCLK = SMCLK, start edge detect
  UBR01 = 0x12;                               // 57600 from 1.048576Mhz
  UBR11 = 0x00;                               //
  UMCTL1 = 0x40;                              // Modulation
  UCTL1 &= ~SWRST;                            // Initalize USART state machine
  IE2 |= URXIE1;                              // Enable USART1 RX interrupt
  
  for (;;)                             
  {
    while (!(UTCTL1 & TXEPT));                // Confirm no TXing before -> LPM3 
    _DINT();                                  // Disable ints for flag test
    _NOP();
    if (UTCTL1 & SSEL0)
      _BIS_SR(LPM3_bits + GIE);               // Enter LPM3 with ints
    else
      _BIS_SR(LPM0_bits + GIE);               // RX'ing char, LPM0, int's active   
  }
}

#pragma vector=UART1RX_VECTOR
__interrupt void usart1_rx (void)
{
  if (IFG2 & URXIFG1)                         // Test URXIFG1
  {
    while (!(IFG2 & UTXIFG1));                // USART1 TX buffer ready?
    TXBUF1 = RXBUF1;                          // RXBUF1 to TXBUF1
    _BIC_SR_IRQ(LPM3_bits);                   // Exit LPM3 on reti
    UTCTL1 |= SSEL0;                          // SSEL0= 1, no RX activity
  }
  else                                        // Start edge
  {
    UTCTL1 &= ~URXSE;                         // Clear URXS signal
    UTCTL1 |= URXSE;                          // Re-enable edge detect
    _BIC_SR_IRQ(SCG1 + SCG0);                 // DCO enabled on reti
    UTCTL1 &= ~SSEL0;                         // SSEL0= 0, RX activity
  }
}

void Set_DCO (void)                           // Subroutine to set DCO
{
  unsigned int Compare, Oldcapture = 0;

  BCSCTL1 |= DIVA_3;                          // ACLK = LFXT1CLK/8
  CCTL2 = CM_1 + CCIS_1 + CAP;                // CAP, ACLK
  TACTL = TASSEL_2 + MC_2 + TACLR;            // SMCLK, cont-mode, clear

  while (1)
  {
    while (!(CCIFG & CCTL2));                 // Wait until capture occurred
    CCTL2 &= ~CCIFG;                          // Capture occurred, clear flag
    Compare = CCR2;                           // Get current captured SMCLK 
    Compare = Compare - Oldcapture;           // SMCLK difference
    Oldcapture = CCR2;                        // Save current captured SMCLK

    if (DELTA == Compare)
    {
      break;                                  // Break loop
    }
    else if (DELTA < Compare)           
    {
      DCOCTL--;
      if (DCOCTL == 0xFF)                     // DCO is too fast, slow it down
        if (BCSCTL1 & 0x07)
          BCSCTL1--;                          // DCO role under?, Sel lower RSEL
        else
          break;                              // Error condition, break loop
    }
    else
    {                      
      DCOCTL++;                               // DCO is too slow, speed it up
      if (DCOCTL == 0x00) 
        if ((BCSCTL1 & 0x07) != 0x07) 
          BCSCTL1++;                          // DCO role over? Sel higher RSEL
        else
          break;                              // Error condition, break loop
    }
  }
  CCTL2 = 0;                                  // Stop CCR2
  TACTL = 0;                                  // Stop Timer_A
}

